/**
 * Copyright (c) 2014-2018 Linagora
 * 
 * This program/library is free software: you can redistribute it and/or modify
 * it under the terms of the GNU Lesser General Public License as published by
 * the Free Software Foundation, either version 2.1 of the License, or (at your
 * option) any later version.
 * 
 * This program/library is distributed in the hope that it will be useful, but
 * WITHOUT ANY WARRANTY; without even the implied warranty of MERCHANTABILITY or
 * FITNESS FOR A PARTICULAR PURPOSE. See the GNU Lesser General Public License
 * for more details.
 * 
 * You should have received a copy of the GNU Lesser General Public License
 * along with this program/library; If not, see http://www.gnu.org/licenses/
 * for the GNU Lesser General Public License version 2.1.
 */
package org.ow2.petals.extension.registry.overlay;

import java.util.Properties;

import org.objectweb.fractal.api.Component;
import org.objectweb.fractal.api.Interface;
import org.objectweb.fractal.api.NoSuchInterfaceException;
import org.objectweb.fractal.api.control.BindingController;
import org.objectweb.fractal.api.control.ContentController;
import org.objectweb.fractal.api.control.IllegalBindingException;
import org.objectweb.fractal.api.control.IllegalContentException;
import org.objectweb.fractal.api.control.IllegalLifeCycleException;
import org.objectweb.fractal.api.factory.InstantiationException;
import org.objectweb.fractal.util.Fractal;
import org.ow2.petals.microkernel.api.configuration.ConfigurationService;
import org.ow2.petals.microkernel.api.configuration.DomainConfiguration.DomainMode;
import org.ow2.petals.microkernel.api.extension.AbstractPetalsExtensionController;
import org.ow2.petals.microkernel.api.extension.PetalsExtensionController;
import org.ow2.petals.microkernel.api.extension.PetalsPreExtensionController;
import org.ow2.petals.microkernel.api.extension.exception.CreationExtensionException;
import org.ow2.petals.microkernel.api.extension.exception.PetalsExtensionException;
import org.ow2.petals.microkernel.api.server.FractalHelper;

import com.ebmwebsourcing.easycommons.log.LoggingUtil;

/**
 * Controller of the Petals pre-extension 'Embedded Petals Registry Overlay'
 * 
 * @author Christophe DENEUX - Linagora
 * 
 */
public class ExtensionController extends AbstractPetalsExtensionController implements PetalsPreExtensionController {

    /**
     * The name of the resources containing the Fractal composite definition associated to the extension 'Embedded
     * Petals Registry Overlay'
     */
    private static final String FRACTAL_COMPOSITE_NAME = "EmbeddedRegistryOverlay";

    /**
     * The name set to the Fractal composite associated to the extension 'Embedded Petals Registry Overlay'
     */
    private static final String EXTENSION_COMPONENT_NAME = "EmbeddedRegistryOverlay";

    /**
     * The name of the extension for logging purposes.
     */
    private final static String EXTENSION_NAME = "Petals Embedded Registry Overlay";

    private final LoggingUtil log = new LoggingUtil(
            java.util.logging.Logger.getLogger(EmbeddedRegistryOverlayService.FRACTAL_COMPONENT_LOGGER_NAME));

    @Override
    public Component createFractalComponent(final Component preExtensionComposite) throws CreationExtensionException {

        try {
            // Create the Fractal component associated to the extension 'Embedded Petals Registry Overlay'
            final Component extensionImpl = FractalHelper.createCompositeComponent(FRACTAL_COMPOSITE_NAME);

            // Add the 'Embedded Petals Registry Overlay' component in the pre-extension composite
            FractalHelper.addComponent(extensionImpl, preExtensionComposite, EXTENSION_COMPONENT_NAME);

            // Next we bind client interfaces of the Petals ESB Embedded registry to the right interfaces
            final ContentController preExtensionCompositeCC = Fractal.getContentController(preExtensionComposite);

            final BindingController preExtensionCompositeBC = Fractal.getBindingController(extensionImpl);

            // As the configuration component is not a sub-component of the pre-extension composite, we link the client
            // interface "configuration" of the embedded registry to the client interface "configuration" of the
            // pre-extension composite.
            final Interface configurationServiceItf = (Interface) preExtensionCompositeCC
                    .getFcInternalInterface(PetalsExtensionController.CONFIGURATION_FRACTAL_ITF_NAME);
            preExtensionCompositeBC.bindFc(ExtensionController.CONFIGURATION_FRACTAL_ITF_NAME, configurationServiceItf);

            return extensionImpl;
        } catch (final InstantiationException e) {
            throw new CreationExtensionException(EXTENSION_NAME, e);
        } catch (final NoSuchInterfaceException e) {
            throw new CreationExtensionException(EXTENSION_NAME, e);
        } catch (final IllegalContentException e) {
            throw new CreationExtensionException(EXTENSION_NAME, e);
        } catch (final IllegalLifeCycleException e) {
            throw new CreationExtensionException(EXTENSION_NAME, e);
        } catch (final IllegalBindingException e) {
            throw new CreationExtensionException(EXTENSION_NAME, e);
        }
    }

    @Override
    public void removeFractalComponent() {
        // TODO Auto-generated method stub
    }

    @Override
    public String getExtensionName() {
        return ExtensionController.EXTENSION_NAME;
    }

    @Override
    public boolean isActivated(final ConfigurationService configurationService) throws PetalsExtensionException {
        final boolean isActivated;

        if (configurationService.getContainersConfiguration().size() == 1) {
            final String registryImplementation = configurationService.getDomainConfiguration()
                    .getSharedAreaImplementation();
            if (registryImplementation != null) {
                this.log.debug("The topology contains one container but a registry implementation is set, so the pre-extension '"
                        + ExtensionController.EXTENSION_NAME + "' is not activated.");
                isActivated = false;
            } else if (configurationService.getDomainConfiguration().getMode() == DomainMode.STATIC) {
                this.log.debug(
                        "The topology contains one container and the registry is not set, but it is a static domain, so the pre-extension '"
                                + ExtensionController.EXTENSION_NAME
                                + "' is not activated because the default standalone implementation will be used.");
                isActivated = false;
            } else {
                final Properties properties = configurationService.getServerProperties();
                final String propertyValue = properties
                        .getProperty(EmbeddedRegistryOverlayService.ACTIVATION_PROPERTY_NAME);
                if (propertyValue != null) {
                    isActivated = Boolean.parseBoolean(propertyValue);
                } else {
                    isActivated = EmbeddedRegistryOverlayService.DEFAULT_ACTIVATION;
                }
            }

        } else {
            this.log.debug("More than one container are declared in the topology, so the pre-extension '"
                    + ExtensionController.EXTENSION_NAME + "' is not activated.");
            isActivated = false;
        }

        return isActivated;
    }

    @Override
    public String[] getDependencies() {
        // No dependency is required
        return null;
    }

    @Override
    public boolean failsPetalsStartup() {
        return true;
    }

}
